@@ -138,11 +138,7 @@ private:
                                      double height_max2,
                                      double tolerance)
         {
-            typedef typename bg::coordinate_system
-                <
-                    Box
-                >::type::units box_units_type;
-
+            using box_units_type = bg::detail::coordinate_system_units_t<Box>;
             std::string const units_str = units2string<box_units_type>();
 
             Box detected;
